import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-n', '--num-types', default=50, type=int, help='The number of types to generate')
parser.add_argument('-f', '--use-friend', default=0, type=int, help='Use hidden-friend operators')
args = parser.parse_args()

n = args.num_types
use_hidden_friend = args.use_friend
print('Generating a header with {} types. Using hidden-friend idiom? {}'.format(n, use_hidden_friend))

def print_operator_equals(outfile, is_friend, i, j):
    if is_friend:
        outfile.write('  friend ')
    outfile.write('bool operator==(const T{}& lhs, const T{}& rhs) {{ return lhs.data == rhs.data; }}\n'.format(i, j))

def print_all_operator_equals(outfile, is_friend, i):
    if i > 0:
        for j in range(i):
            print_operator_equals(outfile, is_friend, i, j)
        outfile.write('\n')
        for j in range(i):
            print_operator_equals(outfile, is_friend, j, i)
        outfile.write('\n')
    print_operator_equals(outfile, is_friend, i, i)

with open('types.h', 'w') as outfile:
    for i in range(n):
        outfile.write('struct T{} {{\n'.format(i))
        outfile.write('  int data;\n')
        if use_hidden_friend:
            print_all_operator_equals(outfile, use_hidden_friend, i)
        outfile.write('};\n\n')

    if not use_hidden_friend:
        for i in range(n):
            print_all_operator_equals(outfile, use_hidden_friend, i)
